import 'package:flutter/material.dart';

/// 通用按钮组件
/// 基本适应任何场景，目前发现不能在[AlertDialog]中的actions中添加，会报错，原因未追查，其余场景皆可使用
class PGButton extends StatelessWidget {
  /// 按钮颜色
  /// 默认为[Colors.transparent]
  final Color color;

  /// 点击事件
  /// 当[disabled]为true时，[onTap]不响应
  final GestureTapCallback onTap;

  /// 点击事件
  /// 当[disabled]为true时，[onTap]不响应
  /// 当[disabled]为true时，[onDisableTap]响应
  final GestureTapCallback onDisableTap;

  /// 双击事件
  /// 当[disabled]为true时，[onDoubleTap]不响应
  final GestureTapCallback onDoubleTap;

  /// 长按事件
  /// 当[disabled]为true时，[onLongPress]不响应
  final GestureLongPressCallback onLongPress;

  /// 按下事件
  /// 当[disabled]为true时，[onTapDown]不响应
  final GestureTapDownCallback onTapDown;

  /// 点击取消事件
  /// 当[disabled]为true时，[onTapCancel]不响应
  final GestureTapCancelCallback onTapCancel;

  /// 左方组件
  /// 显示在标题左方的组件
  final Widget leading;

  /// 右方组件
  /// 显示在标题右方的组件
  final Widget tailing;

  /// 上方组件
  /// 显示在标题上方的组件
  final Widget topping;

  /// 下方组件
  /// 显示在标题下方的组件
  final Widget bottoming;

  /// 标题
  /// 默认为空字符串
  /// 需传递一个字符串，为了方便通过[disabled],[selected]等状态自动切换标题内容或样式
  final String title;

  /// 不可用标题
  /// 默认为空字符串
  /// 在按钮不可用状态下显示的标题，提前设置，通过[disabled]控制
  final String disabledTitle;

  /// 选中标题
  /// 默认为空字符串
  /// 在按钮被选中状态下显示的标题，提前设置，通过[selected]控制
  final String selectedTitle;

  /// 普通标题样式
  /// 设置按钮在正常状态下的样式
  final TextStyle normalStyle;

  /// 不可用样式
  /// 设置按钮在不可用状态下的样式
  final TextStyle disabledStyle;

  /// 选中样式
  /// 设置按钮在被选中状态下的样式
  final TextStyle selectedStyle;

  /// 内容对齐样式
  /// 默认为[AlignmentDirectional.center]
  final AlignmentGeometry alignment;

  /// 内容偏移量
  /// 默认为[EdgeInsets.zero]
  final EdgeInsets contentInsets;

  /// 按钮外部边距
  final EdgeInsets margin;

  /// 高度
  /// 按钮高度，默认为null
  final double height;

  /// 宽度
  /// 按钮宽度，默认为null
  final double width;

  /// 按钮是否可点击
  /// 默认为false，当按钮不可点击时，不会展示水波效果，各种点击手势不会响应
  /// 若设置了不可用状态下的标题、样式、按钮颜色，按钮将自动切换为不可用的样式
  /// 如没有设置不可用状态下的颜色，则将按钮颜色设置为普通状态下颜色的50%
  final bool disabled;

  /// 水波特效是否启用
  /// 默认为false，启用水波特效，设置为true，可关闭，但不会关闭高亮显示
  final bool waveDisabled;

  /// 禁止高亮
  /// 默认为false，启用高亮显示，设置为true，可关闭，但不会关闭水波特效
  final bool highlightDisabled;

  /// 同时禁止水波和高亮特效
  /// 默认为false，同时启用水波和高亮，设置为true，可同时禁用水波和高亮
  final bool effectDisabled;

  /// 水波颜色
  /// 默认为null（官方默认颜色）
  final Color splashColor;

  /// 高亮颜色
  /// 默认为null（官方默认颜色）
  final Color highlightColor;

  /// 禁用颜色
  /// 默认为null，如果设置了[color]，则为[color]的50%，如果没设置[color]，则为[Colors.transparent]
  /// 如果设置了[disabledColor]，则为[disabledColor]
  final Color disabledColor;

  /// 选中颜色
  /// 默认为null，如果设置了[color]，则为[color]
  /// 如果设置了[selectedColor]，则为[selectedColor]
  final Color selectedColor;

  /// 是否选中
  /// 默认为false，设置为true，根据[selectedColor]、[selectedTitle]、[selectedStyle]是否设置来展示不同样式
  final bool selected;

  /// 标题约束
  /// 设置标题组件的约束，可设置[maxWidth]和[maxHeight]来做为约束条件
  final BoxConstraints titleConstraints;

  /// 间距
  /// 可分别设置上下左右组件距离标题的间距
  /// left针对[leading]到标题的间距
  /// right针对[tailing]到标题的间距
  /// top针对[topping]到标题的间距
  /// bottom针对[bottoming]到标题的间距
  final EdgeInsets spacingInsets;

  /// 按钮圆角
  final BorderRadius borderRadius;

  /// 按钮描边
  final Border border;

  /// 按钮阴影
  final List<BoxShadow> boxShadow;

  const PGButton({
    Key key,
    this.color = Colors.transparent,
    this.leading,
    this.tailing,
    this.title = "",
    this.disabledTitle = "",
    this.selectedTitle = "",
    this.normalStyle,
    this.disabledStyle,
    this.selectedStyle,
    this.alignment = AlignmentDirectional.center,
    this.contentInsets = EdgeInsets.zero,
    this.topping,
    this.bottoming,
    this.height,
    this.width,
    this.disabled = false,
    this.onTap,
    this.onDisableTap,
    this.onDoubleTap,
    this.onLongPress,
    this.onTapDown,
    this.onTapCancel,
    this.waveDisabled = false,
    this.splashColor,
    this.highlightColor,
    this.selected = false,
    this.disabledColor,
    this.margin,
    this.highlightDisabled = false,
    this.effectDisabled = false,
    this.selectedColor,
    this.titleConstraints,
    this.spacingInsets = EdgeInsets.zero,
    this.borderRadius,
    this.border,
    this.boxShadow,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Container(
      child: Material(
        child: Ink(
          decoration: BoxDecoration(
            borderRadius: borderRadius,
            border: border,
            boxShadow: boxShadow,
            color: () {
              if (disabled) {
                if (disabledColor == null) {
                  return color;
                } else {
                  return disabledColor;
                }
              } else {
                if (selected) {
                  if (selectedColor == null) {
                    return color;
                  } else {
                    return selectedColor;
                  }
                } else {
                  return color;
                }
              }
            }(),
          ),
          child: InkWell(
            borderRadius: borderRadius,
            splashColor: () {
              // 优先判断是否禁用全部特效
              if (effectDisabled) {
                return Colors.transparent;
              } else {
                // 如果禁用水波特效则设置水波背景色为透明
                if (waveDisabled) {
                  return Colors.transparent;
                } else {
                  return splashColor;
                }
              }
            }(),
            highlightColor: () {
              // 优先判断是否禁用全部特效
              if (effectDisabled) {
                return Colors.transparent;
              } else {
                // 如果禁用高亮则设置高亮色为透明
                if (highlightDisabled) {
                  return Colors.transparent;
                } else {
                  return highlightColor;
                }
              }
            }(),
            child: Container(
              alignment: alignment,
              height: height,
              width: width,
              child: Container(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: () {
                    // 设置上中下组件
                    List<Widget> columnList = [];

                    if (topping != null) {
                      Widget toppingContainer;
                      if (topping is Expanded) {
                        toppingContainer = topping;
                      } else {
                        toppingContainer = Container(
                          child: topping,
                          padding: EdgeInsets.only(bottom: spacingInsets.top),
                        );
                      }
                      columnList.add(toppingContainer);
                    }

                    var row = Row(
                      // 设置中间组件
                      children: () {
                        List<Widget> widgetList = [];

                        // 如果左边组件不为空，则添加左边组件
                        if (leading != null) {
                          Widget leadingContainer;
                          if (leading is Expanded) {
                            leadingContainer = leading;
                          } else {
                            leadingContainer = Container(
                              child: leading,
                              padding:
                                  EdgeInsets.only(right: spacingInsets.left),
                            );
                          }

                          widgetList.add(leadingContainer);
                        }

                        // 在各种状态下都不存在标题时不添加标题组件
                        if (title != "" ||
                            disabledTitle != "" ||
                            selectedTitle != "") {
                          var childContainer = Container(
                            child: Text(
                              () {
                                if (disabled) {
                                  if (disabledTitle == "") {
                                    return title;
                                  } else {
                                    return disabledTitle;
                                  }
                                } else {
                                  if (selected) {
                                    if (selectedTitle == "") {
                                      return title;
                                    } else {
                                      return selectedTitle;
                                    }
                                  } else {
                                    return title;
                                  }
                                }
                              }(),
                              style: () {
                                if (disabled) {
                                  if (disabledStyle == null) {
                                    return normalStyle;
                                  } else {
                                    return disabledStyle;
                                  }
                                } else {
                                  if (selected) {
                                    if (selectedStyle == null) {
                                      return normalStyle;
                                    } else {
                                      return selectedStyle;
                                    }
                                  } else {
                                    return normalStyle;
                                  }
                                }
                              }(),
                              maxLines: 1,
                              overflow: TextOverflow.ellipsis,
                            ),
                            constraints: titleConstraints,
                          );
                          widgetList.add(childContainer);
                        }

                        // 如果右边组件不为空，则添加右边组件
                        if (tailing != null) {
                          Widget tailingContainer;
                          if (tailing is Expanded) {
                            tailingContainer = tailing;
                          } else {
                            tailingContainer = Container(
                              child: tailing,
                              padding:
                                  EdgeInsets.only(left: spacingInsets.right),
                            );
                          }

                          widgetList.add(tailingContainer);
                        }
                        return widgetList;
                      }(),
                      mainAxisSize: MainAxisSize.min,
                    );
                    columnList.add(row);

                    // 如果底部组件不为空，则添加底部组件
                    if (bottoming != null) {
                      Widget bottomingContainer;
                      if (bottoming is Expanded) {
                        bottomingContainer = bottoming;
                      } else {
                        bottomingContainer = Container(
                          child: bottoming,
                          padding: EdgeInsets.only(top: spacingInsets.bottom),
                        );
                      }

                      columnList.add(bottomingContainer);
                    }

                    return columnList;
                  }(),
                ),
                margin: contentInsets,
              ),
            ),
            onTap: () {
              if (!disabled) {
                return onTap;
              } else {
                return onDisableTap;
              }
            }(),
            onDoubleTap: () {
              if (!disabled) {
                return onDoubleTap;
              }
            }(),
            onLongPress: () {
              if (!disabled) {
                return onLongPress;
              }
            }(),
            onTapDown: () {
              if (!disabled) {
                return onTapDown;
              }
            }(),
            onTapCancel: () {
              if (!disabled) {
                return onTapCancel;
              }
            }(),
          ),
        ),
        color: Colors.transparent,
      ),
      margin: margin,
    );
  }
}
